from openai import OpenAI
import os
from time import time
import base64
import os
from io import BytesIO
from typing import Union

import requests
from PIL import Image


def encode_image_base64(image: Union[str, Image.Image]) -> str:
    """encode raw date to base64 format."""
    res = ''
    if isinstance(image, str):
        url_or_path = image
        if url_or_path.startswith('http'):
            FETCH_TIMEOUT = int(os.environ.get('LMDEPLOY_FETCH_TIMEOUT', 10))
            headers = {
                'User-Agent':
                'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 '
                '(KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
            }
            try:
                response = requests.get(url_or_path,
                                        headers=headers,
                                        timeout=FETCH_TIMEOUT)
                response.raise_for_status()
                res = base64.b64encode(response.content).decode('utf-8')
            except Exception:
                pass
        elif os.path.exists(url_or_path):
            with open(url_or_path, 'rb') as image_file:
                res = base64.b64encode(image_file.read()).decode('utf-8')
    elif isinstance(image, Image.Image):
        buffered = BytesIO()
        image.save(buffered, format='PNG')
        res = base64.b64encode(buffered.getvalue()).decode('utf-8')
    return res
  
# initialize the client but point it to TGI
client = OpenAI(
    base_url="http://localhost:8000/v1",
    api_key="EMPTY"
)

model_id = '/mnt/xnncloud/torin/workdirs/internvl_chat_v2_0/lora_3opt_v5_240910_merged'
generation_parameters = {
    "temperature": 0,
    "max_tokens": 128,
    "top_p": 0.01,
    #"max_input_tiles": 12,
    #"repetition_penalty": 1.2,
    "stream": False,
}

def predict(messages):
  #lmdeploy 2s
    return client.chat.completions.create(
        model=model_id, # needs to be /repository since there are the model artifacts stored
        messages=messages,
        **generation_parameters
    )

import decord,cv2
from PIL import Image
video_fn = '/input/projects/op_pipeline/test_high_crf.mp4'
vr = decord.VideoReader(video_fn, num_threads=4)
imgs = vr.get_batch(range(16)).asnumpy()
del vr
resized_imgs = []
for img in imgs:
    resized_imgs.append(Image.fromarray(cv2.resize(img,(448,448))).convert("RGB"))

messages = [
    {"role": "system", 
     "content": "You are a helpful assistant",
    },
    {"role": "user", 
     "content": [
                *({
                    "type": "image_url",
                    "image_url": {
                        'max_dynamic_patch': 1,
                        "url": f'data:image/png;base64,{encode_image_base64(img)}' 
                    },
                } for img in resized_imgs),
        {
            "type": "text",
            "text": "What type of animal is in the image above? What color does it have?"
            # "text": "\nPlease provide the bounding box coordinate of the region this sentence describes: <ref>the bird</ref>"  
        }
    ]},
]

for _ in range(1):
    t1 = time()
    response = predict(messages)
    print(time()-t1)
    print(f"LLM output: {response}")